/*
 * UART.c
 *
 *  Created on: 16.06.2019
 *      Author: alexanderbaumgartner
 *
 *  This file is part of ELoadV1_6.

    ELoadV1_6 is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    ELoadV1_6 is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with ELoadV1_6.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "../tivaHeaders.h"
#include "../Globals.h"
#include "IOs.h"
#include "UART.h"
#include "../DL/CTRLS.h"

#include <float.h>

extern serial_t serial;
extern uint32_t uptime;
extern main_t main_values;

void initUART()
{

    SysCtlPeripheralEnable(SYSCTL_PERIPH_UART0);
    GPIOPinConfigure(GPIO_PA0_U0RX);
    GPIOPinConfigure(GPIO_PA1_U0TX);
    GPIOPinTypeUART(GPIO_PORTA_BASE, GPIO_PIN_0 | GPIO_PIN_1);

    UARTConfigSetExpClk(UART0_BASE, SysCtlClockGet(), UART_BAUD,(UART_CONFIG_WLEN_8 | UART_CONFIG_STOP_ONE |
                                 UART_CONFIG_PAR_NONE));
    IntRegister(INT_UART0,ISR_uart);
    IntEnable(INT_UART0);
    UARTIntEnable(UART0_BASE, UART_INT_RX | UART_INT_RT);

    for(serial.rx_cnt=0;serial.rx_cnt<UART_BUF_SIZE;serial.rx_cnt++)
    {
        serial.recbuf.ui8[serial.rx_cnt]=0;
        serial.sendbuf.ui8[serial.rx_cnt]=0;
    }
    serial.rx_cnt=0;
    serial.tx_cnt=0;
    serial.rx_ok=0;
    serial.checksum1=0;

    //flush uart
    while(UARTCharsAvail(UART0_BASE))
    {
        UARTCharGetNonBlocking(UART0_BASE);
    }
}


void writeUART(uint8_t* buf,uint8_t len)
{
    uint8_t i=0;
    for(i=0;i<len;i++)
    {
        UARTCharPutNonBlocking(UART0_BASE,buf[i]);
        while(UARTBusy(UART0_BASE))
            {;}
    }
}

void ISR_uart()
{
    uint32_t ui32Status;
    uint8_t i=0;
    ui32Status = ROM_UARTIntStatus(UART0_BASE, true);
    UARTIntClear(UART0_BASE, ui32Status);

    while(UARTCharsAvail(UART0_BASE))
    {
        serial.recbuf.ui8[serial.rx_cnt++]=UARTCharGetNonBlocking(UART0_BASE);
    }

    serial.last_rx = uptime;


    if(serial.rx_cnt == UART_RX_SIZE && serial.recbuf.ui8[1] == 0) //Received data (Cyclic transmission)
    {
        //checksum
        serial.checksum1 = 0x00;
        serial.rx_ok=0;
        for( i=2;i<UART_RX_SIZE;i++)
        {
            serial.checksum1 ^=serial.recbuf.ui8[i];
        }
        if(serial.checksum1 == serial.recbuf.ui8[0])
        {
            serial.rx_ok = 0xFF;
            serial.is_configdata = 0x00;
            toggleLeds(PIN_BLUE_LED);
        }
        else
        {
            main_values.status|=ERR_COM_CHECKSUM;
            GPIOPinWrite(GPIO_PORTF_BASE,PIN_GREEN_LED|PIN_BLUE_LED,0x00);
            GPIOPinWrite(GPIO_PORTF_BASE,PIN_RED_LED,0xFF);
        }

    }
    else if(serial.rx_cnt > UART_RX_SIZE_CONF ) //failed to receive data
    {
        serial.rx_cnt=0;
    }
    else if(serial.rx_cnt == UART_RX_SIZE_CONF && serial.recbuf.ui8[1] == 0xFF) //configuration data
    {
        serial.checksum1 = 0x00;
        serial.rx_ok=0;
        for( i=2;i<UART_RX_SIZE_CONF;i++)
        {
            serial.checksum1 ^=serial.recbuf.ui8[i];
        }
        if(serial.checksum1 == serial.recbuf.ui8[0])
        {
            serial.rx_ok = 0xFF;
            serial.is_configdata = 0xFF;
            toggleLeds(PIN_BLUE_LED);
        }
    }

}

void readDataUART()
{
    serial.rx_ok = 0x00;
    serial.rx_cnt=0;
    if(serial.is_configdata == 0x00)
    {
        //read data:
        main_values.set.current = serial.recbuf.f[1];
        main_values.set.power = serial.recbuf.f[2];
        main_values.set.resistance = serial.recbuf.f[3];
        main_values.set.voltage = serial.recbuf.f[4];
        main_values.lim.uvlo = serial.recbuf.f[5];
        main_values.ctrl_mode = serial.recbuf.ui8[2];

        if(serial.recbuf.ui8[3])
            main_values.status = ERR_NO_ERR;
        main_values.enable |= serial.recbuf.ui8[3];
        serial.rx_cnt=0;
        serial.tx_now=0xFF;
    }
    else
    {

        /*  float-index Value
         *  1   Max Power
            2   Max Voltage
            3   Max Current
            4   Max Temp
            5   Input current scale
            6   Output current scale
            7   Input voltage scale
            8   Current ctrl kI
            9   Current ctrl limits
            10  Voltage ctrl kP
            11  Voltage ctrl kI
            12  Cable resistance

         */
        main_values.lim.power = serial.recbuf.f[1];
        main_values.lim.voltage = serial.recbuf.f[2];
        main_values.lim.current = serial.recbuf.f[3];
        main_values.lim.t_sink = serial.recbuf.f[4];
        main_values.conversion.current_in_fac = serial.recbuf.f[5];
        main_values.conversion.current_out_fac = serial.recbuf.f[6];
        main_values.conversion.voltage_fac = serial.recbuf.f[7];

        main_values.curr_con.kP = 0.0f;
        main_values.curr_con.kI = serial.recbuf.f[8]/F_CTRL;
        main_values.curr_con.max = serial.recbuf.f[9];
        main_values.curr_con.min = (-1.0f)*main_values.curr_con.max;

        main_values.volt_con.kI = serial.recbuf.f[11]/F_CTRL;
        main_values.volt_con.kP = serial.recbuf.f[10];
        main_values.volt_con.max = main_values.lim.current;
        main_values.volt_con.min = 0;

        main_values.cable_resistance = serial.recbuf.f[12];

        main_values.init_done = 0xFF;
        main_values.status &= ~(ERR_NO_INIT);
        serial.rx_cnt=0;
        serial.tx_now=0xFF;
    }

}

void writeBackDataUART()
{
    uint8_t i;

    //Calculate filtered values
    //setting read block
    main_values.act_filt.reading_block = 0xFF;

    //Wait to read data (blocked by writing)
    while(main_values.act_filt.writing_block)
    {;}

    main_values.act_filt.current =  (1.0f/100000.0f) * ((float)main_values.act_filt.current_sum / (float)main_values.act_filt.summation_cnt);
    main_values.act_filt.voltage = (1.0f/100000.0f) * ((float)main_values.act_filt.voltage_sum / (float)main_values.act_filt.summation_cnt);
    if(main_values.act_filt.current > 0.0f)
        main_values.act_filt.resistance = main_values.act_filt.voltage / main_values.act_filt.current;
    else
        main_values.act_filt.resistance = FLT_MAX;
    main_values.act_filt.power = (1.0f/100000.0f) * ((float)main_values.act_filt.power_sum / (float)main_values.act_filt.summation_cnt);
    main_values.act_filt.sinkTemp = (1.0f/100.0f) * ((float)main_values.act_filt.sinkTemp_sum / (float)main_values.act_filt.summation_cnt);

    main_values.act_filt.summation_clear = 0xFF;
    //clear reading block
    main_values.act_filt.reading_block = 0x00;

    //write data
    serial.sendbuf.f[1] = main_values.act_filt.current;
    serial.sendbuf.f[2] = main_values.act_filt.power;
    serial.sendbuf.f[3] = main_values.act_filt.resistance;
    serial.sendbuf.f[4] = main_values.act_filt.voltage;
    serial.sendbuf.f[5] = main_values.act_filt.sinkTemp;
    serial.sendbuf.ui8[2] = main_values.status;
    serial.sendbuf.ui8[3] = main_values.ctrl_mode;
    //Checksum:
    serial.sendbuf.ui8[0] = 0x00;
    serial.sendbuf.ui8[1] = 0xAA;
    for(i=2;i<UART_TX_SIZE;i++)
    {
        serial.sendbuf.ui8[0] ^= serial.sendbuf.ui8[i];
        serial.sendbuf.ui8[1] ^= serial.sendbuf.ui8[i];
    }
    writeUART(serial.sendbuf.ui8,UART_TX_SIZE);
}

